package estimateur.gui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.*;
import javax.swing.event.*;

import java.util.*;

import estimateur.controleur.Analyseur;
import estimateur.gui.estimation.SelectionDialog;
import estimateur.gui.financement.*;

/**
 * Fenêtre principale obtenue directement au lancement de 
 * l'application. 
 * 
 *  Elle agit comme le conteneur principal et englobe d'autres
 *  éléments de l'interface graphique. 
 *  
 */

public class FenetrePrincipale extends JFrame {
	
	protected Analyseur analyseur;
	
	private JButton ajout;
	
	private JButton calcul;
	
	private MenuPrincipal menu;
	
	protected JPanel texteBienvenue;
	
	protected JTabbedPane tabbedPane;

	protected JPanel infPanel;
	
	protected Persistance persistance;
	
	public FenetrePrincipale(Analyseur a) {
		
		super();
		
		this.analyseur = a;
		
		//Look and Feel spécial
		try {
			UIManager.setLookAndFeel("com.pagosoft.plaf.PgsLookAndFeel");
		} catch(Exception e) {}
		
		// Quelques propriétés générales de la fenêtre
		this.setTitle("Funding Calculator");
		this.setSize(700, 375);
		//On centre la fenêtre à l'écran
		this.setLocationRelativeTo(null);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setIconImage(imageFromJar("img/logo.png"));
		
		this.persistance = new Persistance(this);
		
		//Barre de menu
		this.menu = new MenuPrincipal(this);
		this.setJMenuBar(this.menu);
		
		//Initialisation
		this.ajout = new LargeButton("img/ajout.png");
		this.ajout.setText("New source");
		this.ajout.addActionListener(new ActionNouvelleSource());
		this.calcul = new LargeButton("img/somme.png");
		this.calcul.setText("Calculate...");
		this.calcul.addActionListener(new ActionCalculer(this));
		
		this.texteBienvenue = new JPanel();
		texteBienvenue.setLayout(new BoxLayout(texteBienvenue, BoxLayout.X_AXIS));
		
		JLabel bienvenueLabel1 = new JLabel(
				"<html><div style=\"text-align:center;\">Welcome to the <b>funding calculator</b>. <br/><br/>" +
				"First, you must add new funding sources with their amount and their reliability. <br/>" +
				"The \"Calculate\" button will give an estimation of how much you can hope to receive." +
				"<br/><br/><br/><br/></div></html>");
		Dimension diP = bienvenueLabel1.getPreferredSize();
		diP.setSize(diP.getWidth(), 400);
		bienvenueLabel1.setMaximumSize(diP);
		

		texteBienvenue.add(Box.createHorizontalGlue());
		texteBienvenue.add(bienvenueLabel1);
		texteBienvenue.add(Box.createHorizontalGlue());
		
		
		// JPanel principal
		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new BorderLayout());
		
			//Boutons de création et de calcul
			JPanel supPanel = new JPanel();
			supPanel.setBorder(BorderFactory.createEmptyBorder(17, 17, 17, 17));
			supPanel.setLayout(new BoxLayout(supPanel, BoxLayout.X_AXIS));
			//mainPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
			supPanel.add(Box.createHorizontalGlue());
			supPanel.add(this.ajout);
			supPanel.add(Box.createHorizontalGlue());
			supPanel.add(Box.createHorizontalGlue());
			supPanel.add(this.calcul);
			supPanel.add(Box.createHorizontalGlue());
			
		tabbedPane = new JTabbedPane();
		//tabbedPane.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);
		
		this.infPanel = new JPanel();
		infPanel.setLayout(new BoxLayout(infPanel, BoxLayout.X_AXIS));
		infPanel.add(texteBienvenue);
		infPanel.add(tabbedPane);
		tabbedPane.setVisible(false);
		
		mainPanel.add(infPanel,BorderLayout.CENTER);
		mainPanel.add(supPanel, BorderLayout.NORTH);
		//mainPanel.add(tabbedPane,BorderLayout.CENTER);
		
		//SourcesPanel PSources = new SourcesPanel();
		//mainPanel.add(PSources);
		
		// Phase finale : affichage
		this.getContentPane().add(mainPanel);
		this.setVisible(true);
		
		// Enregistrement auprès de l'analyseur
		Analyseur.fenetre = this;
		

	}
	
	public void nouvelleSource() {
		
		if (!tabbedPane.isVisible()) {
			texteBienvenue.setVisible(false);
			tabbedPane.setVisible(true);
		}
		
		OngletPanel tab = new OngletPanel();
		//tab.setLayout(new BoxLayout(tab, BoxLayout.Y_AXIS));
		//tab.setBorder(BorderFactory.createEmptyBorder(10, 0, 10, 0));
		//SourcePanel source = new SourcePanel();
		//source.setAlignmentX(Component.LEFT_ALIGNMENT);
		//JTextField nom = source.champsP.nomChamp;
		//JPanel supprPanel = new JPanel();
		//supprPanel.setLayout(new BoxLayout(supprPanel, BoxLayout.X_AXIS));
		
		//LargeButton suppButton = new LargeButton("img/delete2.png");
		//suppButton.setText("Supprimer");
		//supprPanel.add(Box.createHorizontalGlue());
		//supprPanel.add(suppButton);
		//supprPanel.add(Box.createHorizontalGlue());
		//supprPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		// On ajoute un onglet et on se rappelle de son indice
		//tab.add(source);
		//tab.add(supprPanel);
		JTextField nom = tab.source.champsP.nomChamp;
		tabbedPane.addTab(nom.getText(), new ImageIcon(imageFromJar("img/redbtn.png")), tab);
		int indice = tabbedPane.indexOfComponent(tab);
		//Listener de titre
		nom.addCaretListener(new majTitreOnglet(nom));
		tab.suppButton.addActionListener(new ActionSupprimerSource());
		tab.source.initialiserVerificateur(this.tabbedPane);
		//On met le focus sur le nouvel onglet créé
		tabbedPane.setSelectedIndex(indice);
		
		this.menu.suppressionSourceItem.setEnabled(true);
		this.menu.enregistrerItem.setEnabled(true);
		
	}
	
	public void supprimerSource() {
		tabbedPane.remove(tabbedPane.getSelectedIndex());
		if (tabbedPane.getTabCount() == 0) {
			tabbedPane.setVisible(false);
			texteBienvenue.setVisible(true);
			this.menu.suppressionSourceItem.setEnabled(false);
			this.menu.enregistrerItem.setEnabled(false);
		}
	}
	
	public Vector<SourcePanel> getSourcesCorrectes() {
		Vector<SourcePanel> result = new Vector<SourcePanel>();
		int nb = tabbedPane.getTabCount();
		if (nb > 0) {
			for (int i = 0; i < nb; i++){
				SourcePanel source = ((OngletPanel)tabbedPane.getComponentAt(i)).source;
				if (source.verificateur.estOK) result.add(source);
			}
		}
		return result;
	}
	
	public Vector<SourcePanel> getSources() {
		Vector<SourcePanel> result = new Vector<SourcePanel>();
		int nb = tabbedPane.getTabCount();
		if (nb > 0) {
			for (int i = 0; i < nb; i++){
				SourcePanel source = ((OngletPanel)tabbedPane.getComponentAt(i)).source;
				result.add(source);
			}
		}
		return result;
	}
	
	public void afficherErreur(String contenu){
		JOptionPane.showMessageDialog(this, 
				contenu,
				"Error",
				JOptionPane.ERROR_MESSAGE);
	}
	
	public void afficherWarning(String contenu){
		JOptionPane.showMessageDialog(this, 
				contenu,
				"Warning",
				JOptionPane.WARNING_MESSAGE);
	}

	/**
	 * Mettre à jour automatiquement le titre des onglets
	 *
	 */
	class majTitreOnglet implements CaretListener {
		
		JTextField textf; 
		
		public majTitreOnglet(JTextField tf){
			this.textf = tf;
		}

		public void caretUpdate(CaretEvent arg0) {
			String titre = textf.getText();
			if (titre.equals("")) titre = "No name";
			tabbedPane.setTitleAt(tabbedPane.getSelectedIndex(), titre);
			
		}
	
	}
	
	private class ActionNouvelleSource implements ActionListener {
		public void actionPerformed(ActionEvent e){
			nouvelleSource();
		}
	}

	private class ActionSupprimerSource implements ActionListener {
		
		public void actionPerformed(ActionEvent e){
			supprimerSource();
		}
	}
	
	private class ActionCalculer implements ActionListener {
		
		private FenetrePrincipale fenetre;
		
		public ActionCalculer(FenetrePrincipale f){
			this.fenetre = f;
		}
		
		public void actionPerformed(ActionEvent e){
			Vector<SourcePanel> sourcesOK = getSourcesCorrectes();
			if (sourcesOK.size() > 0) {
				// Il y a au moins une source de correcte
				if (tabbedPane.getTabCount() > sourcesOK.size())
					afficherWarning("Some sources are not correctly defined. \n" +
							"Only valid sources will be available for calculation. ");
				SelectionDialog sd = new SelectionDialog(this.fenetre,sourcesOK);
			} else {
				// On ne peut rien calculer
				afficherErreur("Please define at least one valid funding source to \n" +
						"proceed to the calculation!");
			}
		}
	}
	
	
	public static void main(String[] args) {
		

		
		FenetrePrincipale f = new FenetrePrincipale(null);
		
	}

	
	/** 
	 * Obtenir une image à partir de son chemin relatif.
	 * Marche aussi bien avec des fichiers éclatés que dans un jar. 
	 * @param path chemin relatif de l'image
	 * @return objet de l'image demandée
	 */
	public Image imageFromJar(String path){
		return Toolkit.getDefaultToolkit()
		.getImage(getClass().getClassLoader().getResource(path));
	}

}
