package estimateur.gui.estimation;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.awt.geom.Rectangle2D;
import java.text.NumberFormat;

import javax.swing.*;

import java.text.NumberFormat;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;
import java.util.Map.Entry;

import estimateur.gui.financement.ChampsPanel;
import estimateur.modele.Intervalle;
import estimateur.modele.IntervalleMalDefiniException;
import estimateur.modele.Somme;

public class Graphique extends JApplet {
		
	private HashMap<Float, Vector<Intervalle>> alpha_coupes;
	
	private EchelleHorizontale echelleH;
	
	private EchelleVerticale echelleV;
	
	public Graphique(Somme s){
		super();
		
		//POUR RENDRE OPERATIONNEL :
		this.alpha_coupes = s.getAlphaCoupes();
		
		//Sinon pour tester : Exemple de test
		/*
		this.alpha_coupes = new HashMap<Float, Vector<Intervalle>>();
		
		try {
		Vector<Intervalle> alpha0 = new Vector<Intervalle>();
		alpha0.add(new Intervalle(5.0,5.0));
		alpha0.add(new Intervalle(100.0,300.0));
		alpha0.add(new Intervalle(400.0,600.0));
		
		Vector<Intervalle> alpha05 = new Vector<Intervalle>();
		alpha05.add(new Intervalle(5.0,5.0));
		alpha05.add(new Intervalle(120.0,200.0));
		alpha05.add(new Intervalle(210.0,240.0));
		alpha05.add(new Intervalle(260.0,290.0));
		alpha05.add(new Intervalle(420.0,580.0));
		
		Vector<Intervalle> alpha1 = new Vector<Intervalle>();
		alpha1.add(new Intervalle(130.0,190.0));
		alpha1.add(new Intervalle(220.0,230.0));
		
		alpha_coupes.put((float)0.0, alpha0);
		alpha_coupes.put((float)0.5, alpha05);
		alpha_coupes.put((float)1.0, alpha1);
		
		
		} catch(Exception e){
			e.printStackTrace();
		}
		*/
		
		this.echelleH = new EchelleHorizontale(alpha_coupes.get((float)0.0));
		this.echelleV = new EchelleVerticale();
		
	}

	public void paint(Graphics g) {
		
		//Antialiasing
		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);

        Dimension d = getSize();
        int largeur = d.width - 8;
        int hauteur = d.height;
        
        Color cBack = g2.getColor();
        g2.setColor(Color.WHITE);
        g2.fill(new Rectangle2D.Double(0, 0, d.width, d.height));
        g2.setColor(cBack);
        
        this.echelleH.setLargeur(largeur);
        this.echelleV.setHauteur(hauteur);

        //g2.draw(new Rectangle2D.Double(1, 30, d.width-40, d.height-40));
        dessinerAxeVertical(g);
        dessinerAxeHorizontal(g);
        

        //Vector<Intervalle> int1 = alpha_coupes.get((float)0.5);

        //dessinerEtape(g,(float)0.0,(float)0.5,alpha_coupes.get((float)0.0), alpha_coupes.get((float)0.5));
        this.dessinerGraphique(g);
	}
	
	public void dessinerAxeVertical(Graphics g){
        Dimension d = getSize();
        int largeur = d.width - 8;
        int hauteur = d.height;
		
		int espacement = 30;
		
		int axeZeroX = espacement;
		int axeZeroY = hauteur  - 2*espacement;
		int axeUnX = axeZeroX;
		int axeUnY = espacement;
		
		//Légende
		g.drawString("Possibility", 6, 17);
		
		//Dessin de la barre de l'axe
		g.drawLine(axeZeroX, axeZeroY, axeUnX, axeUnY);
		
		NumberFormat nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(2);
		
		//Dessin des graduations
		int decoupage = 10;
		int longueurAxe = axeZeroY - axeUnY + 1;
		
		for (int i = 0; i <= decoupage; i++){
			int ordonneeGrad = (int) (axeZeroY - i*((double)longueurAxe / (double)decoupage));
			//Dessin graduation
			g.drawLine(espacement-3, ordonneeGrad, espacement+3, ordonneeGrad);
			//Affichage nombre
			float nb = i*((float)1/(float)decoupage);
			g.drawString(nf.format(nb), 5, ordonneeGrad+4);
			
		}

	}
	
	public void dessinerAxeHorizontal(Graphics g){
		Dimension d = getSize();
        int largeur = d.width - 8;
        int hauteur = d.height;
		
		int espacement = 30;
		
		int axeZeroX = 2*espacement;
		int axeZeroY = hauteur  - espacement;
		int axeUnX = largeur - espacement;
		int axeUnY = axeZeroY;
		
		// L'axe s'adapte automatiquement :
		// on coupe les zones où la courbe est à zéro (par des - - -)
		// et on optimise l'occupation de l'espace

		NumberFormat nf = NumberFormat.getInstance();
		nf.setMaximumIntegerDigits(2);
		
		Iterator<Intervalle> iteReel = echelleH.intervalles.iterator();
	
		int depart = 60;
		
		int empl = 0;
		for (Intervalle i : echelleH.intervallesDessin){
			dessinerPointille(g,depart);
			g.drawLine(i.getInf().intValue(), axeZeroY, i.getSup().intValue(), axeZeroY);
			depart = i.getSup().intValue()+1;
			//Graduations
			int grad = i.getInf().intValue();
			
			Intervalle intReel = iteReel.next();
			double depReel = ((double)40 * echelleH.unitesCorresp);
			double gradReelle = intReel.getInf();
			while((grad + 30) < i.getSup().intValue()){
				g.drawLine(grad, axeZeroY-3, grad, axeZeroY+3);
				g.drawString(gradAbscisse(gradReelle), grad-7, hauteur-10);
				grad += 40;	
				gradReelle += depReel;
			}
			empl = i.getSup().intValue();
			g.drawString(gradAbscisse(intReel.getSup().intValue()), empl-7, hauteur-10);
			g.drawLine(empl, axeZeroY-3, empl, axeZeroY+3);
		}
		
		g.drawString("Euros", empl+5, hauteur-26);
		
		//Dessin de la barre de l'axe
		//g.drawLine(axeZeroX, axeZeroY, axeUnX, axeUnY);
	}
	
	public void dessinerGraphique(Graphics g){
		
		Set<Entry<Float,Vector<Intervalle>>> set = this.alpha_coupes.entrySet();
		Vector<Float> nivAlpha = new Vector<Float>();
		
		for (Entry<Float,Vector<Intervalle>> elmt : set) {
			nivAlpha.add(elmt.getKey());
		}
		
		java.util.Collections.sort(nivAlpha);
		//On dipose désormais des alpha dans l'ordre
		
		//Niveau 0
		//On relie entre eux les interv de niveau 0
		
		Vector<Intervalle> intVect = alpha_coupes.get(nivAlpha.firstElement());
		int depart = 60;
		int ord = echelleV.getOrdonnee(0.0);
		for (Intervalle sousInt : intVect) {
				int arrivee = echelleH.getAbscisse(sousInt.getInf());
				dessinerLigneGraphique(g,depart,ord,arrivee,ord);
			depart = echelleH.getAbscisse(sousInt.getSup());
		}
		dessinerLigneGraphique(g,depart,ord,depart+20,ord);
		
		//Cas général
		Iterator<Float> iteAlpha = nivAlpha.iterator();
		float alphaInf = (float)0.0;
		float alphaSup = iteAlpha.next();
		while (iteAlpha.hasNext()) {
			alphaInf = alphaSup;
			alphaSup = iteAlpha.next();
			dessinerEtape(g,alphaInf,alphaSup,alpha_coupes.get(alphaInf),
					alpha_coupes.get(alphaSup));
		}
		//On termine pour la dernière ligne
		dessinerEtape(g,alphaSup,1,alpha_coupes.get(alphaSup),
				new Vector<Intervalle>());
		
	}
	
	public void dessinerEtape(Graphics g, float alphaInf, float alphaSup, 
			Vector<Intervalle> intersInf, Vector<Intervalle> intersSup) {
		
		int ordInf = echelleV.getOrdonnee(alphaInf);
		int ordSup = echelleV.getOrdonnee(alphaSup);
		int absInf = 0;
		int absSup = 0;
		
		//On regarde quels sont les supérieurs de chaque inférieur
		for (Intervalle intInf : intersInf) {
			Vector<Intervalle> superieurs = intInf.sousMembres(intersSup);
			absInf = echelleH.getAbscisse(intInf.getInf());
			absSup = echelleH.getAbscisse(intInf.getSup());
			
			if (superieurs.size() == 0) {
				//Il n'y a aucun supérieur
				//On relie les bornes de intInf
				dessinerLigneGraphique(g,absInf,ordInf,absSup,ordInf);	
			} else {
				//On a des supérieurs
				//On relie la borne de celui de gauche
				
				int plusAGauche = echelleH.getAbscisse(superieurs.firstElement().getInf());
				dessinerLigneGraphique(g,absInf,ordInf,plusAGauche,ordSup);
				//On relie la borne de celui tout à droite
				int plusADroite = echelleH.getAbscisse(superieurs.lastElement().getSup());
				dessinerLigneGraphique(g,absSup,ordInf,plusADroite,ordSup);
				//On relie entre eux les sous-intervalles
				
				int depart = -1;
				for (Intervalle sousInt : superieurs) {
					if (depart == -1) {
						
					} else {
						int arrivee = echelleH.getAbscisse(sousInt.getInf());
						dessinerLigneGraphique(g,depart,ordSup,arrivee,ordSup);
						
					}
					depart = echelleH.getAbscisse(sousInt.getSup());
				}
			}
		}
		
	}
	
	public String gradAbscisse(double d){
		String suffixe = "";
		double resultat = d;
		
		if(d >= 1000000000) {
			d = d/1000000000; suffixe = "G";
		} else if (d >= 1000000) {
			d = d/1000000; suffixe = "M";
		} else if (d>=1000) {
			d = d/1000; suffixe = "k";
		} 
		NumberFormat nf = NumberFormat.getInstance();
		if (d<10) {
			nf.setMaximumFractionDigits(2);
		} else if (d <100) {
			nf.setMaximumFractionDigits(1);
		} else {
			nf.setMaximumFractionDigits(0);
		}
		return nf.format(d)+suffixe;
		
	}
	
	/**
	 * Dessiner une ligne en pointillé (40px) sur l'axe des abscisses pour x donné
	 * @param g le graphics
	 * @param x abscisse du point de départ de la ligne 
	 */
	public void dessinerPointille(Graphics g, int x){
		Graphics2D g2d = (Graphics2D) g;
		int hauteur = getSize().height-30;
		Stroke gBack = g2d.getStroke();
		float epaisseur=1; 
		float[] style = {3,3}; 
		g2d.setStroke( new BasicStroke( 
		        epaisseur,
		        BasicStroke.CAP_BUTT,
		        BasicStroke.JOIN_MITER,
		        10.0f,
		        style,
		        0
		        ));
		g2d.drawLine(x, hauteur, x+39, hauteur);
		g2d.setStroke(gBack);

		
	}
	
	public void dessinerLigneGraphique(Graphics g, int x1, int y1, 
			int x2, int y2){
		Graphics2D g2d = (Graphics2D) g;
		int hauteur = getSize().height-30;
		Stroke gBack = g2d.getStroke();
		Color cBack = g2d.getColor();
		g2d.setColor(Color.BLUE);
		g2d.setStroke( new BasicStroke(2));
		g2d.drawLine(x1, y1, x2, y2);
		g2d.setColor(cBack);
		g2d.setStroke(gBack);	
	}
	
	public static void main(String[] args) {
		
		try {
			UIManager.setLookAndFeel("com.pagosoft.plaf.PgsLookAndFeel");
		} catch(Exception e) {}
		
	
		JFrame f = new JFrame();
		f.setSize(650, 378);
		JPanel parent = new JPanel();

		Vector<Point> v = new Vector<Point>();
		v.add(new Point(0,0));
		v.add(new Point(0,1));
		
		
		JApplet g = new Graphique(null);
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		f.getContentPane().add("Center",g);

		
		
		f.setVisible(true);
		
	}
	
	private class EchelleVerticale {
		
		public int zero;
		
		public int un;
		
		public int longueur;
		
		public int hauteur;
		
		public EchelleVerticale(){

			this.zero = 0;
			this.un = 0;
			
			this.longueur = 0;
			this.hauteur = 0;
		}
		
		public void setHauteur(int h){
			this.hauteur = h;
			
			int espacement = 30;
			
			this.zero = hauteur  - 2*espacement;
			this.un = espacement;
			
			this.longueur = zero - un + 1;
		}
		
		public int getOrdonnee(double d){		
			return (int) (this.zero - d*this.longueur);
		}
		
	}
	
	private class EchelleHorizontale {
		
		public Vector<Intervalle> intervalles;
		
		public Vector<Intervalle> intervallesDessin;
		
		//Pour 1px, correspondance en unités réelles
		public double unitesCorresp;
		
		public double nbPixelsCorresp;
		
		private int largeur;
		
		public EchelleHorizontale(Vector<Intervalle> i){
			this.intervalles = i;
			this.largeur = 0;
			this.unitesCorresp = 0;
			this.nbPixelsCorresp = 0.0;
			this.intervallesDessin = new Vector<Intervalle>();
		}
		
		public void setLargeur(int l){
			this.largeur = l;
			this.intervallesDessin.clear();
			
			//Largeur totale des éléments à représenter
			double lTotale = 0.0;
			for (Intervalle i : intervalles) {
				lTotale += i.getSup() - i.getInf() + 1;
			}
			
			//Largeur totale des coupes en - - - -
			int lCoupe = intervalles.size();
			
			//Largeur réellement disponible pour représenter la courbe
			// 60 car l'axe est décalé de 60 par rapport au bord de la fenêtre
			// chaque omission a 40 de largeur
			// marge de droite à 30
			int largeurDispo = this.largeur - 60 - lCoupe*40 - 30;
			
			if (largeurDispo <= 0) largeurDispo = 10;
			
			// Pour 1 unité combien de pixels correspondent sur le graphe
			this.nbPixelsCorresp = (double)largeurDispo / lTotale;
			this.unitesCorresp = lTotale /(double)largeurDispo;

			
			//Correspondance interv réel <-> inter graphe
			int compteur = 60;
			Iterator<Intervalle> ite = intervalles.iterator();
			
			while (ite.hasNext()) {
				Intervalle i = ite.next();
				compteur += 40;
				
				int depl = (int)((i.getSup() - i.getInf()) * nbPixelsCorresp);
				try {
					this.intervallesDessin.add(new Intervalle((double)compteur,(double)compteur+depl));
				} catch (IntervalleMalDefiniException e) {e.printStackTrace();}

				
				compteur += depl + 1;
			}		
			
			
		}
		
		public int getAbscisse(double d){
			int resultat = 0;
			Intervalle intervReel = null;
			int index = 0;
			
			for (Intervalle interv : this.intervalles){
				if (interv.contient(d)) {
					intervReel = interv;
					index = this.intervalles.indexOf(interv);
				}
			}
			
			if (intervReel == null) {
				System.out.println("PROBLEME ABSCISSE : aucun intervalle inférieur " +
						"correspondant !");
				return 0;
			}
			
			//On a récupéré le bon intervalle auquel appartient le point
			// Récupérons celui qui correspond sur le dessin
			Intervalle intervDessin = this.intervallesDessin.get(index);
			double longueurDessin = intervDessin.getSup() - intervDessin.getInf();
			double longueurReelle = intervReel.getSup() - intervReel.getInf();
			int deport = (int) (longueurDessin*(d - intervReel.getInf())/longueurReelle);
			resultat = (int) (intervDessin.getInf() + (double) deport); 
			return resultat;
		}
		
		
		
		
		
	}

}
