package estimateur.gui.financement;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;

import estimateur.modele.*;
/**
 * Il s'agit du JPanel contenant toutes les champs que l'utilisateur 
 * peut remplir pour définir une source financière. 
 *
 */

public class SourcePanel extends JPanel {
	
	public ChampsPanel champsP;
	
	public IncertitudePanel incertitudeP;
	
	private IconButton ajout;
	
	private IconButton suppression;
	
	public JLabel messageEtat;
	
	public Verificateur verificateur;
	
	public SourcePanel(){
		super();
		
		//Bordure invisible
		this.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		//Etirement horizontal
		this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		
		//Initialisation des éléments graphiques
		this.champsP = new ChampsPanel();
		champsP.setMinimumSize(champsP.getPreferredSize());
		this.incertitudeP = new IncertitudePanel();
		this.messageEtat = new JLabel("<html><span style=\"color:red;\">Please define correctly your source. </span></html>");
		
		this.ajout = new IconButton("img/ajout.png");
		this.suppression = new IconButton("img/suppression.png");
		
		//Disposition
		// Sous-panel ouest : champs et incertitude
		JPanel ouest = new JPanel();
		ouest.setLayout(new BoxLayout(ouest, BoxLayout.Y_AXIS));
		
			ouest.add(this.champsP);
			champsP.setAlignmentX(Component.LEFT_ALIGNMENT);
			champsP.setMaximumSize(champsP.getPreferredSize());
			ouest.add(Box.createVerticalStrut(20));
			ouest.add(this.incertitudeP);
			incertitudeP.setAlignmentX(Component.LEFT_ALIGNMENT);
			incertitudeP.setAlignmentY(Component.TOP_ALIGNMENT);
			ouest.add(Box.createVerticalGlue());
				//Panel message d'état
				JPanel etat = new JPanel();
				etat.setLayout(new BoxLayout(etat, BoxLayout.X_AXIS));
				etat.add(Box.createHorizontalGlue());
				etat.add(this.messageEtat);
				messageEtat.setMaximumSize(messageEtat.getPreferredSize());
				messageEtat.setAlignmentX(Component.LEFT_ALIGNMENT);
				etat.add(Box.createHorizontalGlue());
			
			ouest.add(etat);
			etat.setAlignmentX(Component.LEFT_ALIGNMENT);
			ouest.add(Box.createVerticalGlue());
		
		JPanel est = new JPanel();
		est.setLayout(new BoxLayout(est, BoxLayout.Y_AXIS));
		
			est.add(this.ajout);
			est.add(Box.createRigidArea(new Dimension(5,5)));
			est.add(this.suppression);
		
		this.add(ouest);
		//this.add(Box.createHorizontalGlue());
		//this.add(est);
		
	}
	
	public void initialiserVerificateur(JTabbedPane p){
		this.verificateur = new Verificateur(this,p);
	}
	
	public Financement toFinancement(){
		String nom = this.champsP.nomChamp.getText();
		double noyauInf = Double.parseDouble(champsP.noyauInf.getText());
		double noyauSup = Double.parseDouble(champsP.noyauSup.getText());
		double supportInf;
		double supportSup;
		if (champsP.precisionCB.isSelected()) {
			// Les valeurs du support sont les mêmes que celles du noyau
			supportInf = noyauInf;
			supportSup = noyauSup;
		} else {
			// On doit prendre les valeurs du support
			supportInf = Double.parseDouble(champsP.supportInf.getText());
			supportSup = Double.parseDouble(champsP.supportSup.getText());
		}
		int precision = incertitudeP.incertitudeSlider.getValue();
		Incertitude incertitude = Incertitude.TRES_INCERTAIN;
		switch (precision) {
			case 1 : incertitude = Incertitude.TRES_INCERTAIN; break;
			case 2 : incertitude = Incertitude.PLUTOT_INCERTAIN; break;
			case 3 : incertitude = Incertitude.INCERTAIN; break;
			case 4 : incertitude = Incertitude.RAISONNABLE; break;
			case 5 : incertitude = Incertitude.SUR; break;
			default : break;
		}
		
		Financement resultat = null;
		
		try {
			resultat = new Financement(supportInf, supportSup,
			noyauInf, noyauSup,
			incertitude,
			nom);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return resultat;
		
	}
	
	public static void main(String[] args) {
		
		try {
			UIManager.setLookAndFeel("com.pagosoft.plaf.PgsLookAndFeel");
		} catch(Exception e) {}
		
		JPanel p = new SourcePanel();
		JFrame f = new JFrame();
		f.setSize(650, 130);
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		

		
		f.getContentPane().add(p);
		f.setVisible(true);
		
	}

}
