package estimateur.gui.financement;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import java.util.*;

import estimateur.gui.Util;

/**
 * Verificateur va créer des listeners sur chaque 
 * élément modifiable d'un SourcePanel. 
 * Lorsqu'un changement est opéré, une vérification 
 * de la cohérence des données est effectuée, et le 
 * message d'état est mis à jour en conséquence. 
 *
 */

public class Verificateur implements java.io.Serializable {
	
	private SourcePanel source;
	
	private JTabbedPane panel;
	
	public boolean estOK;
	
	public Verificateur(SourcePanel s, JTabbedPane p){
		
		this.source = s;
		this.panel = p;
		this.estOK = false;
		
		EcouteurDeChangement e = new EcouteurDeChangement();
		
		source.champsP.nomChamp.addCaretListener(e);
		source.champsP.noyauInf.addCaretListener(e);
		source.champsP.noyauSup.addCaretListener(e);
		source.champsP.supportInf.addCaretListener(e);
		source.champsP.supportSup.addCaretListener(e);
		
		source.champsP.precisionCB.addActionListener(new CheckboxListener());
		source.incertitudeP.incertitudeSlider
		.addChangeListener(new IncertitudeListener());

		
	}
	
	/**
	 * Procéder à la vérification de chaque donnée entrée. 
	 * @return Le message d'erreur à afficher ("" si correct)
	 */
	public String verifier(){
		String result = "";
		
		String champsVide = "Please define correctly your source. ";
		String nombreIncorrect = "Please enter correct amounts. ";
		String ordreIncorrect = "The amount on the right should be more than the amount on the left.";
		String supportIncorrect = "The range of the \"possible amounts\" must be wider than the \"most plausible\" range! ";
		
		String nomChamp = source.champsP.nomChamp.getText();
		String noyauInf = source.champsP.noyauInf.getText();
		String noyauSup = source.champsP.noyauSup.getText();
		String supportInf = source.champsP.supportInf.getText();
		String supportSup = source.champsP.supportSup.getText();
		
		//Tous les champs doivent être remplis
		if (nomChamp.equals("")
			|| noyauInf.equals("")
			|| noyauSup.equals("")) {
			return champsVide;
		}
		
		if (!source.champsP.precisionCB.isSelected()
			&& (supportInf.equals("")
					|| supportSup.equals(""))) {
			return champsVide;
		}
		
		//Tout est rempli
		//Les sommes doivent être des flottants positifs ou nuls
		Vector<String> champs = new Vector<String>();
		champs.add(noyauInf);
		champs.add(noyauSup);
		if (!source.champsP.precisionCB.isSelected()) {
			champs.add(supportInf);
			champs.add(supportSup);
		}
		
		Iterator<String> it = champs.iterator();
		while (it.hasNext()) {
			try {
				float f = Float.parseFloat(it.next());
				if (f < 0) return nombreIncorrect;
			} catch (Exception e){
				return nombreIncorrect;
			}

		}
		
		// Il y a des floats partout
		// Ils doivent être dans l'ordre
		
		if (Float.parseFloat(noyauInf)>Float.parseFloat(noyauSup))
			return ordreIncorrect;
		
		if (!source.champsP.precisionCB.isSelected() &&
				(Float.parseFloat(supportInf)>Float.parseFloat(supportSup)))
			return ordreIncorrect;
		
		// Dernier point : le noyau doit être inclus dans le support
		
		if (!source.champsP.precisionCB.isSelected()) {
			
			if (Float.parseFloat(noyauInf)<Float.parseFloat(supportInf) 
					|| Float.parseFloat(noyauInf)>Float.parseFloat(supportSup))
				return supportIncorrect;
			
			if (Float.parseFloat(noyauSup)>Float.parseFloat(supportSup) 
					|| Float.parseFloat(noyauSup)<Float.parseFloat(supportInf))
				return supportIncorrect;
			
		}
		
		return result;
	}
	
	/**
	 * Mettre à jour le message d'état. 
	 * Une vérification est automatiquement lancée.
	 *
	 */
	public void majMessage(){
		
		String verif = verifier();
		
		if (verif.equals("")) {
			//La définition de la source est 100% correcte
			
			estOK = true;
			
			source.messageEtat.setText("<html><span style=\"color:green;\">" +
					"Source correctly defined. " +
					"</span></html>");
			int i = panel.getSelectedIndex();
			panel.setIconAt(i, new ImageIcon(Util.call.imageFromJar("img/greenbtn.png")));
			// TODO: sauvegarder
		} else {
			
			// Déf incorrecte
			
			estOK = false;
			
			source.messageEtat.setText("<html><span style=\"color:red;\">" +
					verif +
					"</span></html>");
			int i = panel.getSelectedIndex();
			panel.setIconAt(i, new ImageIcon(Util.call.imageFromJar("img/redbtn.png")));
			
		}
	}
	
	class EcouteurDeChangement implements CaretListener {
		
		public void caretUpdate(CaretEvent arg0) {
			majMessage();	
		}
	
	}
	
	class CheckboxListener implements ActionListener {

		public void actionPerformed(ActionEvent arg0) {
			majMessage();	
		}
	}
	
	class IncertitudeListener implements ChangeListener {
		
	    public void stateChanged(ChangeEvent e) {    
	    	majMessage();
	    }
	}

}
